#include <string.h>
#include <iostream.h>
#include "ASCIIReader.h"

#include "Stack.h"
#include "Hashtable.h"
#include "Object.h"
#include "String.h"
#include "Value.h"



char ASCIIReader::readChar()
{
	skipHeader();
	char c;
	buffer >> c;
	return c;
}

unsigned char ASCIIReader::readUChar()
{
	skipHeader();
	unsigned char c;
	buffer >> c;
	return c;
}

char* ASCIIReader::readCharPtr()
{
	skipHeader();
	char* str = new char[256];
	buffer >> str;
	return str;
}

unsigned char* ASCIIReader::readUCharPtr()
{
	skipHeader();
	unsigned char* str = new unsigned char[256];
	buffer >> str;
	return str;
}

long ASCIIReader::readLong()
{
	skipHeader();
	long l;
	buffer >> l;
	return l;
}

unsigned long ASCIIReader::readULong()
{
	skipHeader();
	unsigned long l;
	buffer >> l;
	return l;
}

short ASCIIReader::readShort()
{
	skipHeader();
	short s;
	buffer >> s;
	return s;
}

unsigned short ASCIIReader::readUShort()
{
	skipHeader();
	unsigned short s;
	buffer >> s;
	return s;
}

int ASCIIReader::readInt()
{
	skipHeader();
	int i;
	buffer >> i;
	return i;
}

unsigned int ASCIIReader::readUInt()
{
	skipHeader();
	unsigned int i;
	buffer >> i;
	return i;
}

float ASCIIReader::readFloat()
{
	skipHeader();
	float f;
	buffer >> f;
	return f;
}

double ASCIIReader::readDouble()
{
	skipHeader();
	double d;
	buffer >> d;
	return d;
}

bool ASCIIReader::readBool()
{
	skipHeader();
	bool b;
	buffer >> b;
	return b;
}

String& ASCIIReader::readString()
{
	skipHeader();
	char str[256];
	buffer >> str;
	return * new String(str);
}

List* ASCIIReader::readList()
{
	char sType[32], sName[32], sEqual[4], sBracket;
	List* newList = new List;
	long size;
	
	buffer >> sType >> size >> sName >> sEqual >> sBracket;

	while ( size-- ) newList->append(readObject());
	
	buffer >> sBracket;

	return newList;
}

Stack* ASCIIReader::readStack()
{
}

Value& ASCIIReader::readValue()
{
	char sType[32], sName[32], sEqual[4], sBracket;
	buffer >> sType >> sName >> sEqual >> sBracket;

	Value& newValue = Value::newByName(String(sType));
	newValue.acceptReader(this);
	
	buffer >> sBracket;
	
	return newValue;
}

Object* ASCIIReader::readObject()
{
	char sType[32], sName[32], sEqual[4];
	unsigned long id;
	
	buffer >> sType >> sName >> sEqual >> id;

	if ( built->contains(id) ) return built->item(id);
	Object* newObject = Object::newByName(String(sType));
	built->insert(id, newObject);
	
	return newObject;
}

Object* ASCIIReader::readRoot()
{
	Object* root = null;

	for( bool init = true; ! buffer.eof(); )
	{
		char sType[32], sEqual[4], sBracket[4], sEndl;
		unsigned long id;
	
		buffer >> sType >> id >> sEqual >> sBracket;

		Object* object = null;
		if ( built->contains(id) ) object = built->item(id);
		else object = Object::newByName(String(sType));
		if ( init ) root = object, init = false;

		object->acceptReader(this);

		buffer >> sBracket >> sEndl;
	}	
	
	return root;
}

void ASCIIReader::skipHeader()
{
	char sType[32], sName[32], sEqual[4];
	buffer >> sType >> sName >> sEqual;
}

ASCIIReader::ASCIIReader(istream& is) : buffer(is)
{
	built = new Hashtable(127);
}

