#include <iostream.h>
#include "ASCIIWriter.h"

#include "String.h"
#include "Value.h"
#include "Object.h"

#include "Stack.h"
#include "List.h"
#include "Iterator.h"



void ASCIIWriter::writeChar(char c)
{
	buffer << indent() << "char"
		<< " name = " << c << endl;
}

void ASCIIWriter::writeUChar(unsigned char c)
{
	buffer << indent() << "unsigned char"
		<< " name = " << c << endl;
}

void ASCIIWriter::writeCharPtr(const char* p)
{
	buffer << indent() << "char*"
		<< " name = " << p << endl;
}

void ASCIIWriter::writeUCharPtr(const unsigned char* p)
{
	buffer << indent() << "unsigned char*"
		<< " name = " << p << endl;
}

void ASCIIWriter::writeLong(long l)
{
	buffer << indent() << "long"
		<< " name = " << l << endl;
}

void ASCIIWriter::writeULong(unsigned long ul)
{
	buffer << indent() << "unsigned long"
		<< " name = " << ul << endl;
}

void ASCIIWriter::writeShort(short s)
{
	buffer << indent() << "short"
		<< " name = " << s << endl;
}

void ASCIIWriter::writeUShort(unsigned short us)
{
	buffer << indent() << "unsigned short"
		<< " name = " << us << endl;
}

void ASCIIWriter::writeInt(int i)
{
	buffer << indent() << "int"
		<< " name = " << i << endl;
}

void ASCIIWriter::writeUInt(unsigned int ui)
{
	buffer << indent() << "unsigned int"
		<< " name = " << ui << endl;
}

void ASCIIWriter::writeFloat(float f)
{
	buffer << indent() << "float"
		<< " name = " << f << endl;
}

void ASCIIWriter::writeDouble(double d)
{
	buffer << indent() << "double"
		<< " name = " << d << endl;
}

void ASCIIWriter::writeBool(bool b)
{
	buffer << indent() << "bool"
		<< " name = " << b << endl;
}

void ASCIIWriter::writeString(const String* s)
{
	buffer << indent() << "String"
		<< " name = " << s->charPtr() << endl;
}

void ASCIIWriter::writeString(const String& s)
{
	writeString(&s);
}

void ASCIIWriter::writeList(const List* l)
{
	String length((long) l->size());
	buffer << indent() << "List " << length.charPtr() << " name = " << endl;
	buffer << indent() << "{" << endl;
	valueIndent++;
	const Object* object;
	for ( Iterator iter(l); object = iter(); writeObject(object) );
	valueIndent--;
	buffer << indent() << "}" << endl;
}

void ASCIIWriter::writeList(const List& l)
{
	writeList(&l);
}

void ASCIIWriter::writeStack(const Stack* s)
{
}

void ASCIIWriter::writeStack(const Stack& s)
{
}

void ASCIIWriter::writeValue(const Value* v)
{
	buffer << indent() << v->typeName() << " name = " << endl << indent() << "{" << endl;
	valueIndent++;
	v->acceptWriter(this);
	valueIndent--;
	buffer << indent() << "}" << endl;
}

void ASCIIWriter::writeValue(const Value& v)
{
	writeValue(&v);
}

void ASCIIWriter::writeObject(const Object* o)
{
	buffer << indent() << o->typeName() << " name = " << o->refAsULong() << endl;
	if (!wasVisited(o)) toVisit->push(o);
}

void ASCIIWriter::writeObject(const Object& o)
{
	writeObject(&o);
}

void ASCIIWriter::writeRoot(const Object* root)
{
	toVisit->push(root);
	while (!toVisit->isEmpty())
	{
		const Object* object = toVisit->pop();
		unsigned long hack = object->refAsULong();
		buffer << indent() << object->typeName() << " " << hack << " = " << endl;
		buffer << indent() << "{" << endl;
		visited->append(object);
		objectIndent++;
		object->acceptWriter(this);
		objectIndent--;
		buffer << indent() << "}" << endl << endl;
	}
}

void ASCIIWriter::writeRoot(const Object& o)
{
	writeRoot(&o);
}


bool ASCIIWriter::wasVisited(const Object* o)
{
	return visited->contains(o);
}

const char* ASCIIWriter::indent()
{
	static char* tabs[6] = { "", "\t", "\t\t", "\t\t\t", "\t\t\t\t", "\t\t\t\t\t" };
	return tabs[(objectIndent + valueIndent) % 6];
}

ASCIIWriter::ASCIIWriter(ostream& os) : buffer(os)
{
	valueIndent = 0;
	objectIndent = 0;
	visited = new List;
	toVisit = new Stack;
}
